import { Injectable, NgZone } from '@angular/core';
import { Observable, BehaviorSubject } from 'rxjs';
import { NativeFirebaseAuth } from './native-firebase';
import { wrapPromise } from './utils';
import './add/run-in-zone';
export var FirebaseAuth = (function () {
    function FirebaseAuth(fbAuth, ngZone) {
        this.fbAuth = fbAuth;
        this.ngZone = ngZone;
        this.$user = new BehaviorSubject('pending');
        new Observable(function (sub) { return fbAuth.onAuthStateChanged(function (user) { return sub.next(user); }, function (err) { return sub.error(err); }, function () { return sub.complete(); }); })
            .runInZone(this.ngZone)
            .subscribe(this.$user);
    }
    /**
     * @param code
     * @returns {Observable<void>} - Returns {@link ActionCodeError} if operation fails.
     */
    FirebaseAuth.prototype.applyActionCode = function (code) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.applyActionCode(code); });
    };
    /**
     * @param code
     * @returns {Observable<ActionCodeInfo>} - Returns {@link ActionCodeError} if operation fails.
     */
    FirebaseAuth.prototype.checkActionCode = function (code) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.checkActionCode(code); });
    };
    /**
     * @param code
     * @param newPassword
     * @returns {Observable<void>} - Returns {@link ConfirmPasswordResetError} if operation fails.
     */
    FirebaseAuth.prototype.confirmPasswordReset = function (code, newPassword) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.confirmPasswordReset(code, newPassword); });
    };
    /**
     *
     * @param email
     * @param password
     * @returns {Observable<User>} - Returns {@link CreateUserWithEmailAndPasswordError} if operation
     *     fails.
     */
    FirebaseAuth.prototype.createUserWithEmailAndPassword = function (email, password) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.createUserWithEmailAndPassword(email, password); });
    };
    /**
     *
     * @param email
     * @returns {Observable<string[]>} - Returns {@link FetchProvidersForEmailError} if operation
     *     fails.
     */
    FirebaseAuth.prototype.fetchProvidersForEmail = function (email) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.fetchProvidersForEmail(email); });
    };
    /**
     * @returns {Observable<UserCredential>} - Returns {@link GetRedirectResultError} if operation
     *     fails.
     */
    FirebaseAuth.prototype.getRedirectResult = function () {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.getRedirectResult(); });
    };
    /**
     * @param email
     * @returns {Observable<void>} - Returns {@link SendPasswordResetEmailError} if operation fails.
     */
    FirebaseAuth.prototype.sendPasswordResetEmail = function (email) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.sendPasswordResetEmail(email); });
    };
    /**
     *
     * @returns {Observable<User>} - Returns {@link SignInAnonymouslyError} if operation fails.
     */
    FirebaseAuth.prototype.signInAnonymously = function () {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.signInAnonymously(); });
    };
    /**
     * @param credential
     * @returns {Observable<User>} - Returns {@link SignInWithCredentialError} if operation fails.
     */
    FirebaseAuth.prototype.signInWithCredential = function (credential) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.signInWithCredential(credential); });
    };
    /**
     * @param token
     * @returns {Observable<User>} - Returns {@link SignInWithCustomTokenError} if operation fails.
     */
    FirebaseAuth.prototype.signInWithCustomToken = function (token) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.signInWithCustomToken(token); });
    };
    /**
     * @param email
     * @param password
     * @returns {Observable<User>} - Returns {@link SignInWithEmailAndPasswordError} if operation
     *     fails.
     */
    FirebaseAuth.prototype.signInWithEmailAndPassword = function (email, password) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.signInWithEmailAndPassword(email, password); });
    };
    /**
     * @param provider
     * @returns {Observable<User>} - Returns {@link SignInWithPopupError} if operation fails.
     */
    FirebaseAuth.prototype.signInWithPopup = function (provider) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.signInWithPopup(provider); });
    };
    /**
     * @param provider
     * @returns {Observable<User>} - Returns {@link SignInWithRedirectError} if operation fails.
     */
    FirebaseAuth.prototype.signInWithRedirect = function (provider) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.signInWithRedirect(provider); });
    };
    FirebaseAuth.prototype.signOut = function () {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.signOut(); });
    };
    /**
     *
     * @param code
     * @returns {Observable<string>} - Returns {@link VerifyPasswordResetCodeError} if operation
     * fails.
     */
    FirebaseAuth.prototype.verifyPasswordResetCode = function (code) {
        var _this = this;
        return wrapPromise(function () { return _this.fbAuth.verifyPasswordResetCode(code); });
    };
    FirebaseAuth.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    FirebaseAuth.ctorParameters = [
        { type: NativeFirebaseAuth, },
        { type: NgZone, },
    ];
    return FirebaseAuth;
}());
//# sourceMappingURL=firebase-auth.service.js.map